//****************************************************************************/
//                 PROPRIETARY INFORMATION OF ATMEL CORPORATION
//
//                     AVR32(R) RISC source code file.
//                       Copyright Atmel Corporation
//                                       
//         The scope of use of the material contained herein is limited
//         strictly to the terms of the LICENSE which grants permission
//         for the use of the material.  Distribution of this material
//            and all copies thereof to any entity other than to the
//           EMPLOYEES of the Licensee, as the term is defined in the
//        LICENSE, is strictly forbidden without a prior written consent
//                            of Atmel Corporation.
//****************************************************************************/
//****************************************************************************/
// 
//  A V R 3 2
//  RTL IMPLEMENTATION, UC3 PLATFORM
//
//  Filename        :  voltage_monitor.v
//  
//  Project number  :  
//  Module name     :  
//  Description     :  A simple model for a voltage monitor
//                     
//
//  Author          :  ronan.barzic@atmel.com
//  Date            :  Thu Feb  3 21:27:00 2011
//
//****************************************************************************/

`timescale 1ns/1ps

module voltage_monitor (/*AUTOARG*/
  // Outputs
  ready, vok,
  // Inputs
  power, enable
  );
  parameter VThreshold = 1000; // mV
  parameter TReady     = 1000; // ns
  parameter TMeasure   = 100; // ns
  

  
`include "startup_parameters.v"  

  input [AVR32_POWER_MSB:0] power;
  input                     enable;
  output                    ready;
  output                    vok;
  
  
  /*AUTOINPUT*/
  
  /*AUTOOUTPUT*/
  
  /*AUTOREG*/
  
  /*AUTOWIRE*/

  // Internal comparator
  
  wire                  internal_comparator;
  
  assign #(TMeasure) internal_comparator = (power>VThreshold); // transport delay
  assign #(TReady,0) ready = enable;
  
  assign vok = enable ? 
               (ready ? internal_comparator : 1'bx) : // We say we don't know what is the value
               1'b0;                                  // of the output when the cell is not ready
                                                      // but we can assume vok = 0 if enable = 0
  
  
  
endmodule // voltage_monitor
/*  
 Local Variables:
 verilog-library-directories:(
 "."
 )
 End:
 */
