`timescale 1ns/1ps

module startup_logic2(/*AUTOARG*/
  // Outputs
  mon1_enable_r, mon2_enable_r, mon3_enable_r, ref1_enable_r,
  ref2_enable_r, regul1_enable_r, regul1_selvdd_r, regul2_enable_r,
  regul2_selvdd_r,
  // Inputs
  clk, porneg, mon1_vok, mon1_ready, mon2_vok, mon2_ready, mon3_vok,
  mon3_ready, ref1_ready, ref2_ready, regul1_ready, regul2_ready
  );

   //parameter VThreshold = 1000; //mV
   
`include "startup_parameters.v" 

  
   input clk;//for testing purposes, internal oscillator
   
   input porneg;//inputs from power on reset
   input mon1_vok;//inputs from voltage monitor 1
   input mon1_ready;
   input mon2_vok;//inputs from voltage monitor 2
   input mon2_ready;
   input mon3_vok;//inputs from voltage monitor 3
   input mon3_ready;
   input ref1_ready;//inputs from voltage reference 1
   input ref2_ready; //inputs from voltage reference 2
   input regul1_ready; //input from voltage regultaror 1
   input regul2_ready; //input from voltage regultaror 2

   output mon1_enable_r;//outputs to voltage monitor 1
   output mon2_enable_r;//outputs to voltage monitor 2
   output mon3_enable_r; //outputs to voltage monitor 3
   output ref1_enable_r;//outputs to voltage reference 1
   output ref2_enable_r; //outputs to voltage reference 2
   output regul1_enable_r;//output to voltage regultaror 1
   output [AVR32_VREG_SELVDD_MSB:0] regul1_selvdd_r;
   output 			    regul2_enable_r;//output to voltage regultaror 2
   output [AVR32_VREG_SELVDD_MSB:0] regul2_selvdd_r;


  /*AUTOREG*/

  /*AUTOWIRE*/

  reg                               mon1_enable_r;
  reg                               mon2_enable_r;
  reg                               mon3_enable_r;
  reg                               ref1_enable_r;
  reg                               ref2_enable_r;
  reg                               regul1_enable_r;
  wire [AVR32_VREG_SELVDD_MSB:0]     regul1_selvdd_r;
  reg                               regul2_enable_r;
  wire [AVR32_VREG_SELVDD_MSB:0]     regul2_selvdd_r;


  reg                               mon1_enable;
  reg                               mon2_enable;
  reg                               mon3_enable;
  reg                               ref1_enable;
  reg                               ref2_enable;
  reg                               regul1_enable;

  reg                               regul2_enable;



  
  reg                               mon1_ready_m;
  reg                               mon1_ready_r;
  reg                               mon1_vok_m;
  reg                               mon1_vok_r;
  reg                               mon2_ready_m;
  reg                               mon2_ready_r;
  reg                               mon2_vok_m;
  reg                               mon2_vok_r;
  reg                               mon3_ready_m;
  reg                               mon3_ready_r;
  reg                               mon3_vok_m;
  reg                               mon3_vok_r;
  reg                               ref1_ready_m;
  reg                               ref1_ready_r;
  reg                               ref2_ready_m;
  reg                               ref2_ready_r;

  reg                               regul1_ready_m;
  reg                               regul1_ready_r;
  reg                               regul2_ready_m;
  reg                               regul2_ready_r;


  reg [STARTUP_FSM_MSB:0]           state_r;
  reg [STARTUP_FSM_MSB:0]          state_nxt;

  reg [AVR32_VREG_SELVDD_MSB:0]     selvdd_r;
  reg [AVR32_VREG_SELVDD_MSB:0]     selvdd;

  
  // Synchronisers for async input signals
  always @(posedge clk or negedge porneg) begin
    if(porneg == 1'b0) begin
      /*AUTORESET*/
      // Beginning of autoreset for uninitialized flops
      mon1_ready_m <= 1'h0;
      mon1_ready_r <= 1'h0;
      mon1_vok_m <= 1'h0;
      mon1_vok_r <= 1'h0;
      mon2_ready_m <= 1'h0;
      mon2_ready_r <= 1'h0;
      mon2_vok_m <= 1'h0;
      mon2_vok_r <= 1'h0;
      mon3_ready_m <= 1'h0;
      mon3_ready_r <= 1'h0;
      mon3_vok_m <= 1'h0;
      mon3_vok_r <= 1'h0;
      ref1_ready_m <= 1'h0;
      ref1_ready_r <= 1'h0;
      ref2_ready_m <= 1'h0;
      ref2_ready_r <= 1'h0;
      regul1_ready_m <= 1'h0;
      regul1_ready_r <= 1'h0;
      regul2_ready_m <= 1'h0;
      regul2_ready_r <= 1'h0;
      // End of automatics
    end
    else begin
      mon1_vok_m   <= mon1_vok;
      mon1_vok_r   <= mon1_vok_m;
      mon1_ready_m <= mon1_ready;
      mon1_ready_r <= mon1_ready_m;

      mon2_vok_m   <= mon2_vok;
      mon2_vok_r   <= mon2_vok_m;
      mon2_ready_m <= mon2_ready;
      mon2_ready_r <= mon2_ready_m;


      mon3_vok_m   <= mon3_vok;
      mon3_vok_r   <= mon3_vok_m;
      mon3_ready_m <= mon3_ready;
      mon3_ready_r <= mon3_ready_m;

      ref1_ready_m   <= ref1_ready;
      ref1_ready_r   <= ref1_ready_m;


      ref2_ready_m   <= ref2_ready;
      ref2_ready_r   <= ref2_ready_m;

      regul1_ready_m <= regul1_ready;
      regul1_ready_r <= regul1_ready_m;

      regul2_ready_m <= regul2_ready;
      regul2_ready_r <= regul2_ready_m;

      
    end
  end

  // Registers for state_r and analog control signals (to ensure glitch free control)
  always @(posedge clk or negedge porneg) begin
    if(porneg == 1'b0) begin
      state_r <= RESET;
      /*AUTORESET*/
      // Beginning of autoreset for uninitialized flops
      mon1_enable_r <= 1'h0;
      mon2_enable_r <= 1'h0;
      mon3_enable_r <= 1'h0;
      ref1_enable_r <= 1'h0;
      ref2_enable_r <= 1'h0;
      regul1_enable_r <= 1'h0;
      regul2_enable_r <= 1'h0;
      selvdd_r <= {(1+(AVR32_VREG_SELVDD_MSB)){1'b0}};
      // End of automatics
    end
    else begin
      state_r <= state_nxt;
      mon1_enable_r   <=   mon1_enable;
      mon2_enable_r   <=   mon2_enable;  
      mon3_enable_r   <=   mon3_enable;  
      ref1_enable_r   <=   ref1_enable;  
      ref2_enable_r   <=   ref2_enable;  
      regul1_enable_r <=   regul1_enable;
      regul2_enable_r <=   regul2_enable;
      selvdd_r <=   selvdd;

    end
  end

  assign vddio1_good = mon1_ready_r && mon1_vok_r;
  assign vddio2_good = mon2_ready_r && mon2_vok_r;

  assign regul1_selvdd_r = selvdd_r;
  assign regul2_selvdd_r = selvdd_r;


  
  always @* begin
    mon1_enable   =  0;
    mon2_enable   =  0;
    mon3_enable   =  0;
    ref1_enable   =  0;
    ref2_enable   =  0;
    regul1_enable =  0;
    regul2_enable =  0;
    selvdd =  0;
    state_nxt = RESET;
    case(state_r)
      RESET: begin
        state_nxt = MONITOR12_ON;
      end

      MONITOR12_ON: begin
        mon1_enable = 1;
        mon2_enable = 1;
        // Wait for both monitor to be ready
        if(mon1_ready_r && mon2_ready_r) begin
          state_nxt = MONITOR12_SELECT; 
        end
        else begin
          state_nxt = MONITOR12_ON;
        end
        
      end

      MONITOR12_SELECT: begin
        mon1_enable = 1;
        mon2_enable = 1;
        if(vddio2_good) begin
          state_nxt = REF2_ON;
        end
        else if (vddio1_good) begin
          state_nxt = REF1_ON;
        end 
        else begin
          // Stay in this state
          state_nxt = MONITOR12_SELECT;
        end
      end
      
      REF1_ON: begin
        ref1_enable = 1;
        if(ref1_ready_r)
          state_nxt = VREG1_ON;
        else
          state_nxt = REF1_ON;
      end


      REF2_ON: begin
        ref2_enable = 1;
        if(ref2_ready_r)
          state_nxt = VREG2_ON;
        else
          state_nxt = REF2_ON;
      end


      VREG1_ON: begin
        ref1_enable = 1;
        regul1_enable = 1;
        mon3_enable = 1;   
        selvdd = VDD_600;
        if( mon3_ready_r) 
          state_nxt = INC;
        else
          state_nxt = VREG1_ON;
          
      end

      VREG2_ON: begin
        ref2_enable = 1;
        regul2_enable = 1;
        mon3_enable = 1; 
        selvdd = VDD_600;
        if( mon3_ready_r) 
          state_nxt = INC;
        else
          state_nxt = VREG2_ON;

      end

      INC: begin
        mon3_enable = 1;   
        ref1_enable= ref1_enable_r;
        ref2_enable= ref2_enable_r;
        regul1_enable = regul1_enable_r;
        regul2_enable = regul2_enable_r; 
        //  We take a shortcut here - The two regulator are not supposed to be ready at the same time
        if((regul1_ready_r || regul2_ready_r) && mon3_ready_r && !mon3_vok_r && (selvdd_r != VDD_1800) && (selvdd_r != VDD_2000)) begin
          selvdd = selvdd_r + 1;
          state_nxt = INC2;
        end        
        else begin
          selvdd = selvdd_r;
          state_nxt = INC;
        end
      end // case: INC
      INC2: begin
        // we wait before
        mon3_enable = 1;   
        ref1_enable= ref1_enable_r;
        ref2_enable= ref2_enable_r;
        regul1_enable = regul1_enable_r;
        regul2_enable = regul2_enable_r;
        selvdd = selvdd_r;
        state_nxt = INC;
      end
      default: begin



        
      end      
    endcase
  end  
endmodule

